<?xml version='1.0' standalone='no'?>
<!DOCTYPE xsl:stylesheet [ <!ENTITY nbsp '&#160;'> ]>
<xsl:stylesheet 
 version="1.0"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:gjs="http://www.georgejames.com">


  <!-- Initial Load of Audit Trail Window
       ==================================
  -->


  <xsl:import href='gjsTree.xsl' />
  <xsl:include href='ServerLink.xsl' />

  <!-- templates divMainHead, folderContent and leafContent are in here -->
  <xsl:include href='AuditTrail_Row.xsl' />


  <!-- Main template -->
  <xsl:template match="/ServerLink/Rows">

    <div id='divMainHead'>
      <xsl:call-template name='divMainHead'/>
    </div>

    <xsl:call-template name='gjs:tree'>
      <xsl:with-param name='divHeadingId' select='"divMessagesHeading"' />
      <xsl:with-param name='divReferenceHeadingId' select='"divMessagesReferenceHeading"' />
      <xsl:with-param name='noItemsFound'>no</xsl:with-param>
    </xsl:call-template>

		<!--	If there were any flagged messages then generate a div that indicates this.  If this div
					exists then the message panel will be automatically opened if the user has previously
					closed it. -->
		<xsl:if test='/ServerLink/Rows[Leaf or Folder]'>
			<div id='divAutoOpenMessagePanel'/>
		</xsl:if>
  
    <!-- Prepare a timer that will look for new flagged messages when it expires -->
    <script>
      parent.frameMain.AuditTrail.g_bPolling=true
      parent.frameMain.AuditTrail.fnSetTimer("<xsl:value-of select='/ServerLink/Rows/Folder[last()]/audit/@atline'/>")
    </script>

  </xsl:template>


  <!-- Populate the audit trail message heading with the appropriate column headings -->
  <xsl:template name='divMainHead'>
    <xsl:variable name='divMainHeadId' select='"divMessagesHeading"'/>

    <table class='layoutTable' width='1000px' gjsTreeId='AuditTrail_List'>
    	<tr>

        <xsl:call-template name='gjs:treeMoresMainHeading' />

        <td width='25px'>
          <input type='checkbox' name='chkAuditPanePolling' checked='true' onchange='AuditTrail.fnTogglePolling()' style='width: 8px; height: 8px;' title='Poll periodically for new messages flagged for me' />
        </td>

        <xsl:call-template name='gjs:treeColumnHeading'>
          <xsl:with-param name='width' select='"130"'/> 
          <xsl:with-param name='columnId'>DateTime</xsl:with-param>
          <xsl:with-param name='caption'>Date/Time</xsl:with-param>         
          <xsl:with-param name='selected'>ascending</xsl:with-param>
          <xsl:with-param name='divMainHeadId'><xsl:value-of select='$divMainHeadId'/></xsl:with-param>
          <xsl:with-param name='divReferenceHeadingId' select='"divMessagesReferenceHeading"' />
					<xsl:with-param name='previousColumnResizable' select='0' />
        </xsl:call-template>

        <xsl:call-template name='gjs:treeColumnHeading'>
          <xsl:with-param name='width' select='"150"'/> 
          <xsl:with-param name='columnId'>Item</xsl:with-param>
          <xsl:with-param name='caption'>Item</xsl:with-param>         
          <xsl:with-param name='divMainHeadId'><xsl:value-of select='$divMainHeadId'/></xsl:with-param>
          <xsl:with-param name='divReferenceHeadingId' select='"divMessagesReferenceHeading"' />
        </xsl:call-template>

        <xsl:call-template name='gjs:treeColumnHeading'>
          <xsl:with-param name='width' select='"20"'/>
          <xsl:with-param name='columnId'>Flag</xsl:with-param>
          <xsl:with-param name='caption'><img style='vertical-align: middle;' src='vcmFlagGrey.gif' height='14px'/></xsl:with-param>         
          <xsl:with-param name='divMainHeadId'><xsl:value-of select='$divMainHeadId'/></xsl:with-param>
          <xsl:with-param name='divReferenceHeadingId' select='"divMessagesReferenceHeading"' />
        </xsl:call-template>

        <xsl:call-template name='gjs:treeColumnHeading'>
          <xsl:with-param name='width' select='"450"'/>
          <xsl:with-param name='columnId'>Message</xsl:with-param>
          <xsl:with-param name='caption'>Message</xsl:with-param>         
          <xsl:with-param name='divMainHeadId'><xsl:value-of select='$divMainHeadId'/></xsl:with-param>
          <xsl:with-param name='divReferenceHeadingId' select='"divMessagesReferenceHeading"' />
        </xsl:call-template>

        <xsl:call-template name='gjs:treeColumnHeading'>
          <xsl:with-param name='width' select='"80"'/> 
          <xsl:with-param name='columnId'>Category</xsl:with-param>
          <xsl:with-param name='caption'>Category</xsl:with-param>         
          <xsl:with-param name='divMainHeadId'><xsl:value-of select='$divMainHeadId'/></xsl:with-param>
          <xsl:with-param name='divReferenceHeadingId' select='"divMessagesReferenceHeading"' />
        </xsl:call-template>

        <xsl:call-template name='gjs:treeColumnHeading'>
          <xsl:with-param name='width' select='"100"'/> 
          <xsl:with-param name='columnId'>UserId</xsl:with-param>
          <xsl:with-param name='caption'>User ID</xsl:with-param>         
          <xsl:with-param name='divMainHeadId'><xsl:value-of select='$divMainHeadId'/></xsl:with-param>
          <xsl:with-param name='divReferenceHeadingId' select='"divMessagesReferenceHeading"' />
        </xsl:call-template>

        <xsl:call-template name='gjs:treeColumnHeading'>
          <xsl:with-param name='width' select='"100"'/> 
          <xsl:with-param name='columnId'>ActionBy</xsl:with-param>
          <xsl:with-param name='caption'>Action By</xsl:with-param>         
          <xsl:with-param name='divMainHeadId'><xsl:value-of select='$divMainHeadId'/></xsl:with-param>
          <xsl:with-param name='divReferenceHeadingId' select='"divMessagesReferenceHeading"' />
        </xsl:call-template>

        <xsl:call-template name='gjs:treeColumnHeading'>
          <xsl:with-param name='width' select='"100"'/> 
          <xsl:with-param name='columnId'>Transaction</xsl:with-param>
          <xsl:with-param name='caption'>Transaction</xsl:with-param>         
          <xsl:with-param name='divMainHeadId'><xsl:value-of select='$divMainHeadId'/></xsl:with-param>
          <xsl:with-param name='divReferenceHeadingId' select='"divMessagesReferenceHeading"' />
        </xsl:call-template>

        <td width='*'></td>
      </tr>
    </table>

  </xsl:template>

  <!-- Attributes for each folder row -->
  <xsl:template name='folderRowAttributes'>
    <xsl:call-template name='leafRowAttributes'/>
  </xsl:template>

  <!-- Attributes for each leaf row -->
  <xsl:template name='leafRowAttributes'>

    <xsl:variable name='onfocus'>
      <xsl:text>fnItemSelect(this);</xsl:text>
      <xsl:text>Class.fnSelect(AuditTrail,this);</xsl:text>
      <xsl:text>Main.fnSetMainHead("divMessagesReferenceHeading");</xsl:text>
    </xsl:variable>

    <!-- Get colour, use the audit trail's category, or the message's category -->
    <xsl:variable name='color'>
      <xsl:choose>
        <xsl:when test='audit'>
          <xsl:value-of select='audit/category/@color' />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select='category/@color' />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:attribute name='tabindex'>0</xsl:attribute>
    <xsl:attribute name='onfocus'><xsl:value-of select='$onfocus' /></xsl:attribute>
    <xsl:attribute name='onclick'><xsl:value-of select='$onfocus' /></xsl:attribute>
    <xsl:attribute name='ondblclick'>AuditTrail.Item.fnOpen()</xsl:attribute>
    <xsl:attribute name='gjsItem'><xsl:value-of select='audit/@item' /></xsl:attribute>
    <xsl:attribute name='gjsAtLine'><xsl:value-of select='audit/@atline' /></xsl:attribute>
    <xsl:attribute name='style'>color: <xsl:value-of select='$color' /></xsl:attribute>
  </xsl:template>

</xsl:stylesheet>
