<?xml version='1.0' encoding='UTF-8'?>
<!DOCTYPE xsl:stylesheet [ <!ENTITY nbsp '&#160;'> ]>
<xsl:stylesheet 
 version="1.0"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:gjs="http://www.georgejames.com">

<!-- Component List
     ==============
-->

<xsl:import href='gjsTree.xsl' />
<xsl:include href='ServerLink.xsl' />


<!-- Information messages -->
<xsl:template match='/ServerLink/Information'>
  <div id='divInfo'>
    <p style='color: red'>
      <img src="vcmWarning.gif"/>&nbsp;&nbsp;
      <xsl:value-of select='@message'/>
    </p>
  </div>
</xsl:template>


<xsl:template match="/ServerLink/Rows">

  <xsl:if test='/ServerLink/Rows/@mores=""'>
    <div id='divFolderHeadingCaption'>
      <xsl:choose>
        <xsl:when test='not(/ServerLink/location/@location="")'>
          Components at <xsl:value-of select='/ServerLink/location/@location' />
        </xsl:when>
        <xsl:otherwise>
          Components
        </xsl:otherwise>
      </xsl:choose>
    </div>
  </xsl:if>

  <div id='divMainHead'>
    <xsl:call-template name='divMainHead' />
  </div>

  <xsl:call-template name='gjs:tree' />

</xsl:template>


<!-- Attach handler to table -->
<xsl:template name='tableAttributes'>
  <xsl:param name='tableId'/>
  <xsl:attribute name='onkeydown'>fnTreeTableKeyDown(Component,this,event);</xsl:attribute>
</xsl:template>


<!-- Column headings for main panel, including 2000px blank trailing one to allow real last one to be resized -->
<xsl:template name='divMainHead'>

  	<table class='layoutTable' width='2000px' gjsTreeId='Component_List'>
  	  <tr>

        <xsl:call-template name='gjs:treeMoresMainHeading' />

	      <td width='25px' class='tableCell'></td>
        
        <xsl:call-template name='gjs:treeColumnHeading'>
          <xsl:with-param name='width' select='"800"'/> 
          <xsl:with-param name='caption'>Component</xsl:with-param>
					<xsl:with-param name='columnId'>Component</xsl:with-param>
          <xsl:with-param name='selected'>ascending</xsl:with-param>
					<xsl:with-param name='previousColumnResizable'>0</xsl:with-param>
        </xsl:call-template>
        
        <xsl:call-template name='gjs:treeColumnHeading'>
          <xsl:with-param name='width' select='"2000"'/> 
        </xsl:call-template>

	      <td width='*'></td>
      </tr>
    </table>

</xsl:template>


<!-- Content for a leaf cell -->
<xsl:template name='leafContent'>
  <xsl:param name='rowId'/>

  <!-- could be either content or applicableFilter at the moment -->
  <xsl:apply-templates>
    <xsl:with-param name='rowId' select='$rowId' />
    <xsl:with-param name='icon' select='@icon' />
    <xsl:with-param name='iconTitle' select='@iconTitle' />
  </xsl:apply-templates>
  
</xsl:template>


<!-- Content for a folder cell -->
<xsl:template name='folderContent'>
  <xsl:param name='rowId'/>

  <!-- could be either component or directory element -->
  <xsl:apply-templates>
    <xsl:with-param name='rowId' select='$rowId' />
    <xsl:with-param name='icon' select='@icon' />
    <xsl:with-param name='iconTitle' select='@iconTitle' />
  </xsl:apply-templates>

</xsl:template>


<!-- If there is a filter in effect then display a row with an info icon -->
<xsl:template match='/ServerLink/Rows/HeadLeaf/applicableFilter'>
  <td class='tableClass'
   width='25'
   title='A filter is in effect, some items may not be displayed'>
    <img src='vcmInfo_l.gif' />
  </td>
  <td class='tableClass'
   width='800'
   colspan='2'
   title='A filter is in effect, some items may not be displayed'
   style='color: blue; font-size: smaller; white-space: nowrap'>
    <xsl:text> Filter: </xsl:text>
    <xsl:value-of select='@applicableFilter'/>
  </td>
</xsl:template>


<!-- A component may be a folder or a leaf, so this may be called from two different contexts -->
<xsl:template match='component'>
  <xsl:param name='rowId'/>
  <xsl:param name='icon'/>
  <xsl:param name='iconTitle'/>

  <xsl:variable name='info'>
    <xsl:call-template name='info'>
      <xsl:with-param name='status' select='@componentStatusText'/>
    </xsl:call-template>
  </xsl:variable>

  <!-- If this is an update then use the old divReferenceHeadingId otherwise use a new unique one -->
  <xsl:variable name='divReferenceHeadingId'>
    <xsl:choose>
      <xsl:when test='../rowUpdate'>
        <xsl:value-of select='../rowUpdate/@divReferenceHeadingId' />
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select='/ServerLink/@runId' />
        <xsl:text>_mainHead</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!-- Icon for Component -->
  <td width='25'>
    <img
     id='{$rowId}icon'
     src='{$icon}'
     title='{$iconTitle}'
     width='25'
     height='16'>
      <xsl:if test='@draggable=1'>
				<xsl:attribute name='onmousedown'>fnItemMouseDown(Component,fnElement("<xsl:value-of select='$rowId'/>main"),event)</xsl:attribute>
				<xsl:attribute name='onmouseup'>fnItemMouseUp(Component,fnElement("<xsl:value-of select='$rowId'/>main"),event)</xsl:attribute>
        <xsl:attribute name='onmousemove'>fnItemMouseMove(Component,fnElement("<xsl:value-of select='$rowId'/>main"),event)</xsl:attribute>
        <xsl:attribute name='ondragstart'>window.event.returnValue=false</xsl:attribute>
        <xsl:attribute name='oncontextmenu'>fnContextMenu("popComponent.<xsl:value-of select='@componentStatus'/>");</xsl:attribute>
      </xsl:if>
    </img>
  </td>

  <!-- Component cell -->
  <td
   width='800px'
   nowrap='nowrap'
   class='treeCell'>
    <span
     nowrap='nowrap'
     id='{$rowId}main'
     tabindex='0'
     title='{$info}'
     gjsRowId='{$rowId}'
     gjsIconId='{$rowId}icon'
     gjsItemType='COMPONENT'
     gjsRelativePath='{@relativePath}'
		 gjsReferenceHeadingId='{$divReferenceHeadingId}'
     gjsItem='{@componentType}.{@componentName}'
     gjsObjectVersion='{@objectVersion}'
     gjsContextObjectVersion='{@contextObjectVersion}'
		 gjsLocation='{/ServerLink/location/@location}'
     gjsShowObv='{@showObv}'
		 gjsShortName='{@shortName}'
     gjsParentId='{/ServerLink/Rows/@parentId}'
     gjsPtext='{@ptext}'
     gjsItemStatus='{@componentStatus}'
     gjsCtype='{@componentType}'
     gjsCname='{@componentName}'
     gjsFullName='{@fullComponentName}'>
<!-- This colspan is needed for a leaf but not for a folder-->
      <xsl:if test='@objectVersion=""'>
        <xsl:attribute name='colspan'>10</xsl:attribute>
      </xsl:if>
      <xsl:if test='@draggable=1'>
        <xsl:attribute name='onmousedown'>fnItemMouseDown(Component,this,event)</xsl:attribute>
        <xsl:attribute name='onmouseup'>fnItemMouseUp(Component,this,event)</xsl:attribute>
        <xsl:attribute name='onmousemove'>fnItemMouseMove(Component,this,event)</xsl:attribute>
        <xsl:attribute name='oncontextmenu'>fnContextMenu("popComponent.<xsl:value-of select='@componentStatus'/>");</xsl:attribute>
        <xsl:attribute name='gjsOnSpace'>fnItemSpace(Component,event.target,event)</xsl:attribute>
      </xsl:if>
<!-- keep the following in step with htmlCret^%vc760 -->
      <xsl:value-of select='@component'/>

      <xsl:if test='@objectVersion!="" and @showObv=1'>
        <span style='color: graytext'>
          <xsl:text>&nbsp;/</xsl:text>
          <xsl:value-of select='substring-after(@objectVersion,"/")'/>
        </span>
      </xsl:if>
<!-- -->
      <!-- Padding for short component names, so that they can be selected -->
      <xsl:value-of select='substring("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;",string-length(@component))'/>
    </span>
  </td>

</xsl:template>


<!-- Non-draggable folder for a directory -->
<xsl:template match='directory'>
  <xsl:param name='rowId'/>

  <!-- If this is an update then use the old divReferenceHeadingId otherwise use a new unique one -->
  <xsl:variable name='divReferenceHeadingId'>
    <xsl:choose>
      <xsl:when test='../rowUpdate'>
        <xsl:value-of select='../rowUpdate/@divReferenceHeadingId' />
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select='/ServerLink/@runId' />
        <xsl:text>_mainHead</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name='onfocus'>
    <xsl:text>Main.fnSetMainHead("</xsl:text>
    <xsl:value-of select='$divReferenceHeadingId' />
    <xsl:text>");</xsl:text>
  </xsl:variable>

  <!-- Icon for Directory -->
  <td width='25px'>
    <img
     src='{../@icon}'
     title='{../@iconTitle}'
     width='25px'
     height='16px'/>
  </td>

  <!-- directory cell -->
  <td class='treeCell'
   width='600px'
   nowrap='nowrap'>
    <span
     nowrap='nowrap'
     id='{$rowId}main'
     gjsRowId='{$rowId}'
     gjsRelativePath='{@relativePath}'
     gjsObjectVersion=''
     tabindex='0'
     onfocus='{$onfocus}'
     title='{../@iconTitle}'>
     <xsl:value-of select='@directoryName'/>
    </span>
  </td>

</xsl:template>


<!-- Tool tip -->
<xsl:template name='info'>
  <xsl:param name='status'/>
  
  <xsl:text>Status: </xsl:text>
  <xsl:value-of select='$status'/>
  <xsl:if test='@objectVersion!="" and @showObv=1'>
    <xsl:text>
Object: </xsl:text>
    <xsl:value-of select='@objectVersion'/>
  </xsl:if>
  <xsl:text>
Component: </xsl:text>
  <xsl:value-of select='translate(@fullComponentName," ","&nbsp;")'/>
</xsl:template>


<!-- Sub-folder method -->
<xsl:template name='subFolderMethod'>
  <xsl:text>Component.fnListSub</xsl:text>
</xsl:template>


<!-- Sub-folder arguments -->
<xsl:template name='subFolderArguments'>
	<xsl:param name='rowId'/>

  <xsl:text>"</xsl:text>
  <xsl:value-of select='/ServerLink/location/@location'/>
  <xsl:text>"+delimiter+"</xsl:text>
  <xsl:value-of select='/ServerLink/Rows/@componentMaskSequence'/>
  <xsl:text>"+delimiter+$('#lstSelect').scombobox('val')+delimiter+Select.fnFilters()</xsl:text>
	<xsl:text>+delimiter+getAttr(fnElement("</xsl:text><xsl:value-of select='$rowId'/><xsl:text>main"),"gjsRelativePath")</xsl:text>
  <xsl:text>+delimiter+gjsTree.fnMores(gCurrentCell)</xsl:text>
  <xsl:text>+delimiter+getAttr(fnElement("</xsl:text><xsl:value-of select='$rowId'/><xsl:text>main"),"gjsObjectVersion")</xsl:text>

</xsl:template>


<!-- Process mru list -->
<xsl:template match='/ServerLink/mrus'>
  <xsl:call-template name='toolbarSelect'/>
  <xsl:call-template name='toolbarFilters'/>
</xsl:template>


<!-- Selection toolbar -->
<xsl:template name='toolbarSelect'>
  <div id='divToolbarSelect'>
    <table
     cellspacing='1px'
     cellpadding='1px'
     border='0px'>
      <tr>

        <td
         width='186px'>
          <select
           style='width: 186px;'
           size='1'
           id='lstSelect' 
           name='lstSelect' 
           title='Component selection specification. Can include * and ? wildcards. Accepts multiple specs separated by ;' 
           value='{/ServerLink/mrus/@currentSelection}' 
           class='input'>
            <xsl:apply-templates select="/ServerLink/filters/filter" mode='list'/>
            <xsl:apply-templates select="/ServerLink/mrus/mru" mode='list'/>
          </select>
        </td>

        <td>
          <input
           type='submit'
           class='button'
           style='height: 22px; width: 60px; padding-top: 0px;'
           id='cmdOK'
           title='Click to list components that match the pattern in the field on the left, filtered according to the state of the buttons on the right'
           value='Refresh'
           onclick="Select.fnSelect($('#lstSelect').scombobox('val'));return false;"/>
        </td>
      </tr>
    </table>
  </div>
</xsl:template>


<!-- Filters toolbar 
Keep logic in sync with code in list^%vc760 to blank out inapplicable options
-->
<xsl:template name='toolbarFilters'>
  <div id='divToolbarFilters'>
    <table
     width='10'
     cellspacing='1'
     cellpadding='1'
     border='0'>
      <tr>

        <xsl:call-template name='button'>
          <xsl:with-param name='imageId'>btnEmpty</xsl:with-param>
          <xsl:with-param name='imageDown'>vcm_blank.gif</xsl:with-param>
          <xsl:with-param name='imageUp'>vcm_blank.gif</xsl:with-param>
          <xsl:with-param name='currentState' select='""'/>
          <xsl:with-param name='title' select='""'/>
        </xsl:call-template>

        <xsl:choose>
          <xsl:when test='/ServerLink/location/@isLibrary!="0"'>
            <xsl:call-template name='button'>
              <xsl:with-param name='imageId'>btnCheckedOut</xsl:with-param>
              <xsl:with-param name='imageDown'>vcm_blank.gif</xsl:with-param>
              <xsl:with-param name='imageUp'>vcm_blank.gif</xsl:with-param>
              <xsl:with-param name='currentState' select='""'/>
              <xsl:with-param name='title' select='""'/>
            </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name='button'>
              <xsl:with-param name='imageId'>btnCheckedOut</xsl:with-param>
              <xsl:with-param name='imageDown'>vcmComponentGreenButtonDown.gif</xsl:with-param>
              <xsl:with-param name='imageUp'>vcmComponentGreenButtonUp.gif</xsl:with-param>
              <xsl:with-param name='currentState' select='@checkedOut'/>
              <xsl:with-param name='title' select='"Show checked out components"'/>
            </xsl:call-template>
          </xsl:otherwise>
        </xsl:choose>

        <xsl:call-template name='button'>
          <xsl:with-param name='imageId'>btnRegistered</xsl:with-param>
          <xsl:with-param name='imageDown'>vcmComponentAmberButtonDown.gif</xsl:with-param>
          <xsl:with-param name='imageUp'>vcmComponentAmberButtonUp.gif</xsl:with-param>
          <xsl:with-param name='currentState' select='@registered'/>
          <xsl:with-param name='title' select='"Show registered components"'/>
        </xsl:call-template>

        <xsl:call-template name='button'>
          <xsl:with-param name='imageId'>btnError</xsl:with-param>
          <xsl:with-param name='imageDown'>vcm_blank.gif</xsl:with-param>
          <xsl:with-param name='imageUp'>vcm_blank.gif</xsl:with-param>
          <xsl:with-param name='currentState' select='""'/>
          <xsl:with-param name='title' select='""'/>
        </xsl:call-template>

        <xsl:choose>
          <xsl:when test='/ServerLink/location/@isLibrary!="0"'>
            <xsl:call-template name='button'>
              <xsl:with-param name='imageId'>btnUnregistered</xsl:with-param>
              <xsl:with-param name='imageDown'>vcm_blank.gif</xsl:with-param>
              <xsl:with-param name='imageUp'>vcm_blank.gif</xsl:with-param>
              <xsl:with-param name='currentState' select='""'/>
              <xsl:with-param name='title' select='""'/>
            </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name='button'>
              <xsl:with-param name='imageId'>btnUnregistered</xsl:with-param>
              <xsl:with-param name='imageDown'>vcmComponentRedButtonDown.gif</xsl:with-param>
              <xsl:with-param name='imageUp'>vcmComponentRedButtonUp.gif</xsl:with-param>
              <xsl:with-param name='currentState' select='@unregistered'/>
              <xsl:with-param name='title' select='"Show components that are not registered to Deltanji in this location, or that belong to objects in the Error state"'/>
            </xsl:call-template>
          </xsl:otherwise>
        </xsl:choose>

        <xsl:call-template name='button'>
          <xsl:with-param name='imageId'>btnSuperseded</xsl:with-param>
          <xsl:with-param name='imageDown'>vcm_blank.gif</xsl:with-param>
          <xsl:with-param name='imageUp'>vcm_blank.gif</xsl:with-param>
          <xsl:with-param name='currentState' select='""'/>
          <xsl:with-param name='title' select='""'/>
        </xsl:call-template>

        <xsl:call-template name='button'>
          <xsl:with-param name='imageId'>btnAllVersions</xsl:with-param>
          <xsl:with-param name='imageDown'>vcm_blank.gif</xsl:with-param>
          <xsl:with-param name='imageUp'>vcm_blank.gif</xsl:with-param>
          <xsl:with-param name='currentState' select='""'/>
          <xsl:with-param name='title' select='""'/>
        </xsl:call-template>

      </tr>
    </table>
  </div>
</xsl:template>


<xsl:template match='/ServerLink/mrus/mru' mode='list'>
  <option value='{@selection}'>
    <xsl:if test="/ServerLink/mrus/@currentSelection=@selection">
      <xsl:attribute name='selected'>selected</xsl:attribute>
    </xsl:if>
    <xsl:value-of select="@selection" />
  </option>
</xsl:template>


<xsl:template match='/ServerLink/filters/filter' mode='list'>
  <option value='{@selection}'>
    <xsl:if test="/ServerLink/mrus/@currentSelection=@selection">
      <xsl:attribute name='selected'>selected</xsl:attribute>
    </xsl:if>
    <xsl:value-of select="@caption" />
  </option>
</xsl:template>


<xsl:template name='button'>
  <xsl:param name='imageId'/>
  <xsl:param name='imageDown'/>
  <xsl:param name='imageUp'/>
  <xsl:param name='currentState'/>
  <xsl:param name='title'/>
  <xsl:param name='filterText'/>
        <td
         align='left'
         title='{$title}'
         style='padding-top: 3px;'
         width='1'>
          <!-- Hide blank buttons -->
          <xsl:if test='($imageUp="vcm_blank.gif")'>
            <xsl:attribute name='style'>display: none;</xsl:attribute>
          </xsl:if>
          <img
           id='{$imageId}'
           onmouseover='Button.fnMouseOver(this)'
           onmouseout='Button.fnMouseOut(this)'
           onclick='Button.fnClick(this)'
           ondblclick='fnRefresh()'
           gjsImageDown='{$imageDown}'
           gjsImageUp='{$imageUp}'
           tabindex='0'
           height='25'
           width='25'>
            <xsl:choose>
              <xsl:when test='$currentState="1"'>
                <xsl:attribute name='src'><xsl:value-of select='$imageDown'/></xsl:attribute>
                <xsl:attribute name='gjsState'>1</xsl:attribute>
              </xsl:when>
              <xsl:otherwise>
                <xsl:attribute name='src'><xsl:value-of select='$imageUp'/></xsl:attribute>
                <xsl:attribute name='gjsState'><xsl:value-of select='$currentState'/></xsl:attribute>
              </xsl:otherwise>
            </xsl:choose>
          </img>
        </td>
</xsl:template>


<!-- Swallow the mrus -->
<xsl:template match='/ServerLink/mrus/mru' />


<!-- Swallow the filters -->
<xsl:template match='/ServerLink/filters/filter' />


</xsl:stylesheet>
