<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- Copyright 2004 George James Software Limited -->

<!-- This stylesheet transforms an xml formatted version of a Cache.cpf file
     to the [Datasets] and [Namespaces] sections of a genuine Cache.cpf file -->


<xsl:output method="text" indent="no"/>
<xsl:strip-space elements="*"/>


<xsl:template match="cpf">
  <xsl:variable name="lastUsedDatabaseNumber" select="@lastUsedDatabaseNumber"/>
  <xsl:variable name="lastUsedNamespaceNumber" select="@lastUsedNamespaceNumber"/>
  <xsl:apply-templates/>
</xsl:template>


<xsl:template match="cpf/databases">
<xsl:text>
[Datasets]
</xsl:text>
  <xsl:apply-templates select='database'/>
</xsl:template>


<xsl:template match="cpf/databases/database">
  <xsl:text>DataSet_</xsl:text>
  <xsl:value-of select='position()+../../@lastUsedDatabaseNumber'/>
  <xsl:text>=</xsl:text>
  <xsl:value-of select='@name'/>
  <xsl:text>,</xsl:text>
  <xsl:value-of select='@directory'/>
  <xsl:text>,</xsl:text>
  <xsl:value-of select='@remoteSystem'/>
  <xsl:text>,</xsl:text>
  <xsl:text>0,</xsl:text>
  <xsl:text>P,</xsl:text>
  <xsl:call-template name='yesNoDisplayToLogical'>
    <xsl:with-param name='value' select='@readOnly'/>
    <xsl:with-param name='default'>0</xsl:with-param>
    </xsl:call-template>
  <xsl:text>,</xsl:text>
  <xsl:call-template name='yesNoDisplayToLogical'>
    <xsl:with-param name='value' select='@mountRequiredForStartup'/>
    <xsl:with-param name='default'>0</xsl:with-param>
  </xsl:call-template>
  <xsl:text>
</xsl:text>
</xsl:template>


<xsl:template match="cpf/namespaces">
  <xsl:text>
[Namespaces]
</xsl:text>
  <xsl:apply-templates select='namespace' mode='namespace'/>
  <xsl:apply-templates select='namespace' mode='namespaceMap'/>
</xsl:template>


<xsl:template match="cpf/namespaces/namespace" mode="namespace">
  
  <!-- Set up defaults for each attribute -->
  <xsl:variable name='database'>
    <xsl:choose>
      <xsl:when test='not(@database)'>
        <xsl:value-of select='@name'/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select='@database'/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name='routineDatabase'>
    <xsl:choose>
      <xsl:when test='not(@routineDatabase)'>
        <xsl:value-of select='$database'/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select='@routineDatabase'/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name='globalDatabase'>
    <xsl:choose>
      <xsl:when test='not(@globalDatabase)'>
        <xsl:value-of select='$database'/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select='@globalDatabase'/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name='systemGlobalDatabase'>
    <xsl:choose>
      <xsl:when test='not(@systemGlobalDatabase)'>
        <xsl:text>CACHESYS</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select='@systemGlobalDatabase'/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name='systemRoutineDatabase'>
    <xsl:choose>
      <xsl:when test='not(@systemRoutineDatabase)'>
        <xsl:text>CACHESYS</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select='@systemRoutineDatabase'/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name='cacheLibrary'>
    <xsl:choose>
      <xsl:when test='not(@cacheLibrary)'>
        <xsl:text>CACHELIB</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select='@cacheLibrary'/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name='temporaryStorage'>
    <xsl:choose>
      <xsl:when test='not(@temporaryStorage)'>
        <xsl:text>CACHETEMP</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select='@temporaryStorage'/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:text>Nsp_</xsl:text>
  <xsl:value-of select='position()+../../@lastUsedNamespaceNumber'/>
  <xsl:text>=</xsl:text>
  <xsl:value-of select='@name'/>
  <xsl:text>,</xsl:text>
  <xsl:value-of select='$globalDatabase'/>
  <xsl:text>,</xsl:text>
  <xsl:value-of select='$systemGlobalDatabase'/>
  <xsl:text>,</xsl:text>
  <xsl:value-of select='$routineDatabase'/>
  <xsl:text>,</xsl:text>
  <xsl:value-of select='$systemRoutineDatabase'/>
  <xsl:text>,</xsl:text>
  <xsl:value-of select='$cacheLibrary'/>
  <xsl:text>,</xsl:text>
  <xsl:value-of select='$temporaryStorage'/>
  <xsl:text>
</xsl:text>
</xsl:template>


<xsl:template match="cpf/namespaces/namespace" mode="namespaceMap">
  <xsl:apply-templates select='globalMap' mode='namespaceMap'/>
  <xsl:apply-templates select='routineMap' mode='namespaceMap'/>
</xsl:template>


<xsl:template match='cpf/namespaces/namespace/globalMap' mode='namespaceMap'>
  <xsl:if test='position()=1'>
    <xsl:text>
[NspMap.</xsl:text>
    <xsl:value-of select='../@name' />
    <xsl:text>]
</xsl:text>
  </xsl:if>

  <xsl:variable name='replicationLockDatabase'>
    <xsl:choose>
      <xsl:when test='not(@replicationLockDatabase)'>
        <xsl:value-of select='@database'/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select='@replicationLockDatabase'/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!-- Set variable subscriptMaps to 1 if this globalMap has subscriptMaps otherwise 0 -->
  <xsl:variable name='subscriptMaps'>
    <xsl:choose>
      <xsl:when test='count(subscriptMap)'>
        <xsl:text>1</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>0</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name='collation'>
    <xsl:choose>
      <xsl:when test='count(subscriptMap)=0'>  <!-- No subscript maps -->
        <xsl:text>-1</xsl:text>
      </xsl:when>
      <xsl:when test='not(@collation)'>  <!-- Cache Standard -->
        <xsl:text>5</xsl:text>
      </xsl:when>
      <xsl:when test='@collation="Cache Standard"'>
        <xsl:text>5</xsl:text>
      </xsl:when>
      <xsl:when test='@collation="Cache Standard String"'>
        <xsl:text>133</xsl:text>
      </xsl:when>
      <xsl:when test='@collation="ISM Pre-6.2"'>
        <xsl:text>0</xsl:text>
      </xsl:when>
      <xsl:when test='@collation="ISM 6.2->6.4"'>
        <xsl:text>1</xsl:text>
      </xsl:when>
      <xsl:when test='@collation="Ipsum/Cobra"'>
        <xsl:text>2</xsl:text>
      </xsl:when>
      <xsl:when test='@collation="Ipsum/Cobra-2"'>
        <xsl:text>4</xsl:text>
      </xsl:when>
      <xsl:when test='@collation="ISM Pre-6.2 string"'>
        <xsl:text>128</xsl:text>
      </xsl:when>
      <xsl:when test='@collation="ISM 6.2->6.4 string"'>
        <xsl:text>129</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:message terminate="yes">
          Context: cpf/namespaces/namespace/globalMap/@collation
          Invalid value: <xsl:value-of select='@collation'/>
        </xsl:message>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:text>GloMap_</xsl:text>
  <xsl:value-of select='position()'/>
  <xsl:text>=</xsl:text>
  <xsl:value-of select='@mask'/>
  <xsl:text>,</xsl:text>
  <xsl:value-of select='@database'/>
  <xsl:text>,</xsl:text>
  <xsl:value-of select='$replicationLockDatabase'/>
  <xsl:text>,</xsl:text>
  <xsl:value-of select='$subscriptMaps'/>
  <xsl:text>,</xsl:text>
  <xsl:value-of select='$collation'/>
  <xsl:text>,</xsl:text>
  <xsl:apply-templates select='replication' mode='namespaceMap'/>
  <xsl:text>
</xsl:text>

  <xsl:apply-templates select='subscriptMap' mode='namespaceMap'/>

</xsl:template>


<xsl:template match='cpf/namespaces/namespace/globalMap/replication' mode='namespaceMap'>
  <xsl:if test='position()>1'>
    <xsl:text>~</xsl:text>
  </xsl:if>
  <xsl:value-of select='@database'/>
</xsl:template>


<xsl:template match='cpf/namespaces/namespace/globalMap/subscriptMap' mode='namespaceMap'>
  <xsl:text>GloSLM_</xsl:text>
  <xsl:value-of select='../@mask'/>
  <xsl:text>_</xsl:text>
  <xsl:value-of select='position()'/>
  <xsl:text>=</xsl:text>
  <xsl:value-of select='@range'/>
  <xsl:text>~</xsl:text>
  <xsl:value-of select='@database'/>
  <xsl:text>~</xsl:text>
  <xsl:choose>
    <xsl:when test='count(replication)=0'>1</xsl:when>
    <xsl:otherwise>0</xsl:otherwise>
  </xsl:choose>
  <xsl:text>~</xsl:text>
  <xsl:apply-templates select='replication' mode='namespaceMap'/>
  <xsl:text>
</xsl:text>
</xsl:template>


<xsl:template match='cpf/namespaces/namespace/globalMap/subscriptMap/replication' mode='namespaceMap'>
  <xsl:if test='position()>1'>
    <xsl:text>~</xsl:text>
  </xsl:if>
  <xsl:value-of select='@database'/>
</xsl:template>


<xsl:template match='cpf/namespaces/namespace/routineMap' mode='namespaceMap'>
  <xsl:if test='position()=1 and count(../globalMap)=0'>
    <xsl:text>
[NspMap.</xsl:text>
    <xsl:value-of select='../@name' />
    <xsl:text>]
</xsl:text>
</xsl:if>

  <xsl:variable name='routineType'>
    <xsl:choose>
      <xsl:when test='not(@routineType)'>
        <xsl:text>ALL</xsl:text>
      </xsl:when>
      <xsl:when test='@routineType="ALL"'>
        <xsl:text>ALL</xsl:text>
      </xsl:when>
      <xsl:when test='@routineType="INC"'>
        <xsl:text>INC</xsl:text>
      </xsl:when>
      <xsl:when test='@routineType="INT/BAS"'>
        <xsl:text>INT</xsl:text>
      </xsl:when>
      <xsl:when test='@routineType="INT"'>
        <xsl:text>INT</xsl:text>
      </xsl:when>
      <xsl:when test='@routineType="BAS"'>
        <xsl:text>INT</xsl:text>
      </xsl:when>
      <xsl:when test='@routineType="MAC"'>
        <xsl:text>MAC</xsl:text>
      </xsl:when>
      <xsl:when test='@routineType="OBJ"'>
        <xsl:text>OBJ</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:message terminate="yes">
          Context: cpf/namespaces/namespace/routineMap/@routineType
          Invalid value: <xsl:value-of select='@routineType'/>
        </xsl:message>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:text>RouMap_</xsl:text>
  <xsl:value-of select='position()'/>
  <xsl:text>=</xsl:text>
  <xsl:value-of select='@mask'/>
  <xsl:text>,</xsl:text>
  <xsl:value-of select='$routineType'/>
  <xsl:text>,</xsl:text>
  <xsl:value-of select='@database'/>
  <xsl:text>
</xsl:text>
</xsl:template>


<!-- Transform yesNo datatype from Display format to Logical -->
<xsl:template name="yesNoDisplayToLogical">
  <xsl:param name="value"/>
  <xsl:param name="default"/>
  <xsl:choose>
    <xsl:when test="$value='yes'">1</xsl:when>
    <xsl:when test="$value='no'">0</xsl:when>
    <xsl:when test="not($value)"><xsl:value-of select='$default'/></xsl:when>
    <xsl:otherwise>
      <xsl:message terminate="yes">
        Invalid yes/no datatype format: <xsl:value-of select='$value'/>
      </xsl:message>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


</xsl:stylesheet>
