//vc/ gjs - Generic Form Manager - Modal functions
//vc/ Copyright 2003 George James
//vc/ Object:    gjsModalForm.js


// Tab-strip class
// This is the modal version of TabStrip from gjsForm.js
TabStrip = {}

TabStrip.selectedFrame = []
TabStrip.selectedTab = []

TabStrip.fnSelect = function(oTab) {

	const formId = Form.selectedFormId

	if (TabStrip.selectedTab[formId]) {
		TabStrip.selectedTab[formId].tabIndex = -1
		$(TabStrip.selectedTab[formId]).removeClass().addClass('tabStripTab')
		$(TabStrip.selectedFrame[formId]).removeClass().addClass('tabPanelHidden')
	}
	//
	TabStrip.selectedTab[formId] = oTab
	TabStrip.selectedTab[formId].tabIndex = 0

	// Save a pointer to the <div> element that holds the contents of the selected frame
	const frameDiv = getAttr(oTab,"gjsDivId")
	const selectedFrame = $(frmMain).find('#' + frameDiv)[0]
	TabStrip.selectedFrame[formId] = selectedFrame

	// Make the selected frame visible and highlight its tab
	$(TabStrip.selectedFrame[formId]).removeClass().addClass('tabPanelVisible')
	$(TabStrip.selectedTab[formId]).removeClass().addClass('tabStripTabSelected')
}


TabStrip.fnKeyDown = function(oTab,event) {

	//Left-arrow
	if ((event.keyCode == 37) && !event.altKey) {
		const oAdjacentTab = oTab.previousElementSibling
		if (oAdjacentTab != null) {
			oAdjacentTab.focus()
		}
		Form.fnEventStop(event)
	}
	
	// Right-arrow
	if ((event.keyCode == 39) && !event.altKey) {
		const oAdjacentTab = oTab.nextElementSibling;
		if ((oAdjacentTab != null) && !($(oAdjacentTab).hasClass('tabStripTabNone'))) {
			oAdjacentTab.focus()
		}
		Form.fnEventStop(event)
	}
}


// =======================================================
// Modal Forms
// The following methods are used by the modal version
// of <gjs:form>
// =======================================================

var frmMain // frmMain will point to the form that currently has focus

// Modal Form class
Form = {}

Form.changed = []          // Dirty flag for each form
Form.oSelectedRow = []     // Table control selected row
Form.frmMain = []          // Array of pointers to the <form> element in each modal form
Form.selectedFormId = null // Currently selected form


// Create a modal form
Form.create = (oResult, top, left, width, height) => {

	const formId = oResult.formBody.getAttribute('data-formid')
	const divFormId = 'divForm' + formId
	const divFormElement = '#' + divFormId

	// Insert a div at the end of the main form after divOuter
	// this is the container for the form
	const html = '<div id="' + divFormId + '" style="display:none; overflow:hidden" class="gjaxWindow hidden" onmousedown="Form.focus(\''+formId+'\')"></div>'
	$(html).insertAfter('#divOuter')

	// Copy content of <Form> element from oResult into the above <div> element
	if (oResult.getElementById('Form') === null) alert("<Form> element missing from stylesheet")
	ServerLink.fnUpdate(oResult,'Form',divFormId)

	// Enable the above <div> as a moveable and resizable modal window 
	$(divFormElement).myOwnDialog({
		title: $('#formBody'+formId).attr('title'),
		movable: true,
		resizable: true,
		pos_y: top,
		pos_x: left,
		width: width,
		height:height,
		bg_color: 'buttonface',
		border: '1px solid buttonshadow'
	})

	$(divFormElement).myOwnDialog('open')
	$(divFormElement).show()

	// Pointer to the <form> element within the form.  frmMain will be set to this when
	// the form has focus.
	Form.frmMain[formId] = document.getElementById('formMain'+formId)

	// Bring this form to the front and give it focus (and set frmMain)
	Form.focus(formId)

	// Run the onload attribute of this gjaxWindow's formBody which is where the
	// value of <gjs:form onload='...' /> gets stored
	try {
		eval($('#formBody'+formId).attr('onload'))
	}
	catch(error) {
		alert("<gjs:form> onload error: " + error)
	}

	return formId
}


// Close and destroy a modal form
Form.destroy = () => {
	const divFormId = 'divForm' + Form.selectedFormId
	const divFormElement = '#' + divFormId
	$(divFormElement).myOwnDialog("close")
	$(divFormElement).remove()
	delete Form.changed[Form.selectedFormId]
	delete Form.oSelectedRow[Form.selectedFormId]
	delete Form.frmMain[Form.selectedFormId]
	Form.selectedFormId = null
} 


// Bring form to front and give it focus
Form.focus = (formId) => {

	const divFormElement = '#divForm' + formId

	// Track which form has focus (this is a bit of a hack)
	//const formId = gjaxWindow.id.substring(7)
	Form.selectedFormId = formId

	// Make frmMain available as a pointer to the current form
	frmMain = Form.frmMain[formId]

	// Send each popup window back by one (from 50 down to 40)
	$('.gjaxWindow').each( function() {
		if (this.style.zIndex > 40) {
			this.style.zIndex = this.style.zIndex - 1
		}
	})
	
	// Bring this window to the fore
	$(divFormElement).css('zIndex', 50)
}


Form.fnHelp = (event) => {
	if (typeof(SelectedClass.fnHelp)=='function') {
		SelectedClass.fnHelp()	
	} 
	else {
		winHelp=window.open('WebDoc/docs/index.html','vcmHelp','')
		winHelp.focus()
	}	
	Form.fnEventStop(event)
}


Form.fnChanged = () => {
	Form.changed[Form.selectedFormId] = true
}


Form.fnClose = () => {
	if (Form.changed[Form.selectedFormId]) {
		const ok = confirm("There are unsaved changes. \nAre you sure you want to close this window?")
		if (!ok) return
	}
	Form.destroy(Form.selectedFormId)
}


Form.fnDelete = function() {
	if (!confirm("Are you sure you want to delete this record?")) return

	// Invoke application specific delete method
	SelectedClass.fnItemDelete()
}


	// Invoked when changes have been saved
Form.fnSaved = (thisClass) => {
	Form.changed = false
	Form.fnRefresh(thisClass)
}


Form.fnRefresh = function(thisClass, bFullReload) {
	var bRefreshed = false
	var oEl

	if (typeof(bFullReload) == 'undefined') bFullReload=false
	
	if (SelectedClass != thisClass) return // In case main window has changed to some other screen

	// If full reload requested, locate +/- cell of selected item and run its reload
	if (bFullReload) {
		if (typeof(SelectedClass.selectedCells) != "undefined") {
			if (SelectedClass.selectedCells.length > 0) {
				oEl = fnElement(SelectedClass.selectedCells[0])
				oEl = oEl.parentNode.parentNode.parentNode.parentNode.parentNode
				if (oEl.tagName == 'TBODY') {
				fnToggleExpand(fnElement(getAttr(oEl, "gjsClickId")))
				bRefreshed = true
				}
			}
		}
	}
  
  if (bRefreshed) return

  // If fnRefresh method is available and acceptable then use it
  if ((SelectedClass.fnRefresh) && (!bFullReload)) bRefreshed = fnRefreshSelected()
  if (bRefreshed) return

  // If we couldn't do a selective refresh, try a full one
  if (SelectedClass.fnList)	SelectedClass.fnList()
}


	// ------------------------- Row selection and manipulation ---------------

	// Row Change Manager
	// The *second* row forms the template for new rows, it should be blank and hidden
	// The last row should be blank but not hidden.  When a value is entered in the
	// last row, then a new row should be added using the *second* row as a template.
	// [The first row is used to set the column widths and is normally empty].
Form.fnRowChanged = function(oThisRow) {

	Form.fnChanged()

	// Is this the last row?
	if (getAttr(oThisRow,"gjsLastRow") == '0') return
	setAttr(oThisRow,"gjsLastRow","0")

	// Clone *second* row (including its events) as new last row
	const oTableBody = $(oThisRow).parent()
	const oNewRow = $(oTableBody).children("tr:nth-of-type(2)")
	$(oNewRow).clone(true).attr("gjsLastRow","1").css("display","table-row").appendTo(oTableBody)
}


	// Toggle row selection
Form.fnRowSelect = function(oRow,event) {

	const formId = Form.selectedFormId

	// Deselect current row
	if (Form.oSelectedRow[formId] == oRow) {
		$(Form.oSelectedRow[formId]).removeClass().addClass('row')
		Form.oSelectedRow[formId] = null
	}
	else { 
		if (Form.oSelectedRow[formId]) $(Form.oSelectedRow[formId]).removeClass().addClass('row')
		Form.oSelectedRow[formId] = oRow
		$(Form.oSelectedRow[formId]).removeClass().addClass('selectedRow')
	}

	event.cancelBubble = true
}


	// Move selected row up one
Form.fnRowUp = function() {

	const formId = Form.selectedFormId

	if (!Form.oSelectedRow[formId]) return

	if (Form.oSelectedRow[formId].rowIndex < 2) return

	const oBody = Form.oSelectedRow[formId].parentNode
	const oPreviousRow = Form.oSelectedRow[formId].previousElementSibling
	oBody.insertBefore(Form.oSelectedRow[formId], oPreviousRow)

	Form.fnChanged()
}


	// Move selected row down one
Form.fnRowDown = function() {

	const formId = Form.selectedFormId

	if (!Form.oSelectedRow[formId]) return

	if (!Form.oSelectedRow[formId].nextElementSibling) return

	const oBody = Form.oSelectedRow[formId].parentNode
	const oNextRow = Form.oSelectedRow[formId].nextElementSibling
	oBody.insertBefore(oNextRow,Form.oSelectedRow[formId])

	Form.fnChanged()
}


	// Reset row selection
Form.fnRowReset = function() {

	const formId = Form.selectedFormId

	if (formId !== null) {
		if (Form.oSelectedRow[formId]) {
			$(Form.oSelectedRow[formId]).removeClass().addClass('row')
			Form.oSelectedRow[formId] = null
		}
	}
}


	// ------------------------- Event handlers ---------------

Form.fnKeyDown = function(event) {

	const formId = Form.selectedFormId

	// Selected row keyboard events
	if (formId !== null) {
		if (Form.oSelectedRow[formId]) {
			if (event.keyCode==38) {
				var elem=event.srcElement || event.target
				Form.fnRowUp()
		
				// At least on IE, if we don't refocus to the element we can't use the keyboard to move a row up more than one place
				elem.focus()
		
				Form.fnEventStop(event)
			}
			if (event.keyCode==40) {
				Form.fnRowDown()
				Form.fnEventStop(event)
			}
		}
	}

	// Escape key and the form is of type ok/cancel
	// Move focus there to ensure any onchange fires for currently-focused field, then simulate a click on the Cancel
	if (event.keyCode == 27) {
		var elCancel = document.getElementById("cmdCancel" + formId)
		if (elCancel){ 
			elCancel.focus()
			elCancel.click() 
			Form.fnEventStop(event)
		}
	}

	// The user may accidentally try to use some keyboard shortcuts
	// Ctrl+N
	if ((event.keyCode == 78) && (event.ctrlKey)) Form.fnEventStop(event)

	// ctrl O for File/New
	if ((event.keyCode == 79) && (event.ctrlKey)) Form.fnEventStop(event)

	// F3 - Search
	if (event.keyCode == 114) { 
		Utility.fnDebug()
		return Form.fnEventStop(event) 
	}
}


  // Bulletproof method for stopping events from bubbling
Form.fnEventStop = function(event) {
	event.keyCode = 0 // Required to stop F5 key
	event.returnValue = false
	event.cancelBubble = true
	if (event.preventDefault) event.preventDefault()
	if (event.stopPropagation) event.stopPropagation()
	return false
}


window.onkeydown = Form.fnKeyDown
