<?xml version="1.0" encoding="utf-8"?>

<!--  Usage Instructions
      Import (do not include) gjsModalForm.xsl in an xsl template to generate a form.  Use the following
      elements to draw and populate a form:
        <gjs:form>
					<gjs:information>
						<gjs:tabStrip>
							<gjs:tabStripTab>
						<gjs:frame>
						<gjs:columnWidths>
							<gjs:column>
						<gjs:fields>
							<gjs:input> etc
					
      Override any of the following named templates to provide custom behaviour:
				<gjs:buttonBarBefore> to add custom buttons to the left of the button bar
				<gjs:buttonBar> to override the complete button bar
				<gjs:buttonBarAfter> to add custom buttons to the right of the button bar 
-->

<xsl:stylesheet 
 version="1.0"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:exslt="http://exslt.org/common"
 xmlns:msxsl="urn:schemas-microsoft-com:xslt"
 xmlns:gjs="http://www.georgejames.com"
 exclude-result-prefixes="exslt msxsl">

<xsl:output
    method="html"
    doctype-public="XSLT-compat"
    omit-xml-declaration="yes"
    encoding="UTF-8"
    indent="yes" />

<xsl:include href='ServerLink.xsl' />

<!-- This variable is unique for each ServerLink response. It is used to 
	 ensure all elements in the form are unique as there can be multiple
	 instances of the form on one page -->
<xsl:variable name='formId'>
	<xsl:value-of select='concat("_",/ServerLink/@runId)' />
</xsl:variable>


<!-- Apply templates that are in the main body of the document and capture the output
     in the string variable sForm.  This executes the normal xsl processing of the
     main document. -->
<xsl:variable name="sForm">
	<xsl:apply-templates/>
</xsl:variable>

<!-- Convert the sForm variable from a string to a node-set and process the node-set.
 Use mode="form" to make this processing distinct from that of the main document.
 Capture the output in a variable called sForm1. -->
<xsl:variable name="sForm1">
  <xsl:choose>
    <xsl:when test="function-available('exslt:node-set')">
      <xsl:apply-templates select="exslt:node-set($sForm)" mode="form" />
    </xsl:when>
    <xsl:when test="function-available('msxsl:node-set')">
      <xsl:apply-templates select="msxsl:node-set($sForm)" mode="form" />
    </xsl:when>
  </xsl:choose>
</xsl:variable>

<!-- Convert the sForm1 variable from a string to a node-set and process the node-set.
 Use mode="controls" to translate the controls. -->
<xsl:template match="/">
  <xsl:choose>
    <xsl:when test="function-available('exslt:node-set')">
      <xsl:apply-templates select="exslt:node-set($sForm1)" mode="controls" />
    </xsl:when>
    <xsl:when test="function-available('msxsl:node-set')">
      <xsl:apply-templates select="msxsl:node-set($sForm1)" mode="controls" />
    </xsl:when>
  </xsl:choose>
</xsl:template>

<!-- Copy Verbatim
     =============
     The following template copies everthing verbatim, but applies any relevant templates along the
     way.  This way, elements like <gjs:input> can be used wherever you need them.
     Note this is done for normal mode, form mode and controls mode.
-->
<xsl:template match="@*|*">
  <xsl:copy>
    <xsl:apply-templates select="@*|node()" />
  </xsl:copy>
</xsl:template>

<xsl:template match="@*|*" mode="form">
  <xsl:copy>
    <xsl:apply-templates select="@*|node()" mode="form"/>
  </xsl:copy>
</xsl:template>

<xsl:template match="@*|*" mode="controls">
  <xsl:copy>
    <xsl:apply-templates select="@*|node()" mode="controls"/>
  </xsl:copy>
</xsl:template>

<!-- ======================================================================= -->

<xsl:include href='gjsControlsTemplates.xsl'/>

<xsl:template match="gjs:form" mode="form">

  <!-- Everything inside this div, but not this div itself, is copied into the modal form body -->
  <div id="Form"> 

    <xsl:variable name='width'>
      <xsl:choose>
        <xsl:when test='@width and (translate(@width,"px","")=@width)'>
          <xsl:value-of select='concat(@width,"px")'/>
        </xsl:when>
        <xsl:when test='@width'>
          <xsl:value-of select='@width'/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>0px</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>


		<!-- The user's preferences for this session -->
<!--		
    <gjs:preferences>
			<xsl:if test='@language'>
				<xsl:attribute name='language'><xsl:value-of select='@language'/></xsl:attribute>
			</xsl:if>
			<xsl:if test='@dictionary'>
				<xsl:attribute name='dictionary'><xsl:value-of select='@dictionary'/></xsl:attribute>
			</xsl:if>
		</gjs:preferences>
-->
  		
    <!-- Load gjsForm styles -->
    <link rel='stylesheet' href='gjsForm.css' />
    <link rel="stylesheet" href="modalbox.css" />

  		<!-- Set defaults for the calendar controls now -->
      <!--<script>
        $(function(){
          $.datepicker.setDefaults({
          <xsl:choose>
            <xsl:when test='user/@locale="US"'>
              dateFormat: 'mm/dd/yy' 
            </xsl:when>
            <xsl:otherwise>
              dateFormat: 'dd/mm/yy' 
            </xsl:otherwise>
          </xsl:choose>
          });
        });
      </script>-->


    <!-- Application specific script file -->
    <script src='{@jsFile}' />
	

    <!-- Form.fnAutoSize call is delayed by a timer because Chrome tends to misreport window size during onload -->
    <div
      title='{@title}'
      class='formBody'
  	  id='formBody{$formId}'
	    style='padding: 5px; overflow: visible; border-top: 1px solid buttonshadow;'
      onclick='Form.fnRowReset()'
      onload='{@onload}'
     >

      <!-- Form -->
      <form 
        name='formBody' 
        data-formid='{$formId}'
        id='formMain{$formId}'
        onsubmit='SelectedClass.fnItemSaveClose("{$formId}"); return false;'
        style='overflow: visible;'>
    
        <xsl:apply-templates mode='form'/>

        <!-- Button bar -->
        <div id='divButtonBar{$formId}' class='buttonBar'>

          <table class='formLayoutTable' border='0' id='buttonBarTable{$formId}'>

            <tr>
              <td align='right' style='min-widthx: {$width};'>	
                <xsl:call-template name='gjs:buttonBarBefore'/>
                <xsl:call-template name='gjs:buttonBar' />	
                <xsl:call-template name='gjs:buttonBarAfter'/>
              </td>
            </tr>

            <tr>
              <td> </td>
            </tr>
          </table>
        </div>

  		</form>
	  </div>
  </div>
</xsl:template>


<!-- Custom buttons to left of main button bar -->
<!-- Override to provide content -->
<xsl:template name='gjs:buttonBarBefore'>
</xsl:template>


<!-- Standard Button bar -->
<!-- Can be overridden if required -->
<xsl:template name='gjs:buttonBar'>
	<xsl:if test='not(@save="no")'>
		<button
			type='button' 
			class='button' 
			id='cmdSave{$formId}'
			onclick='SelectedClass.fnItemSave()'
			accesskey='S'>
			<xsl:if test='@save="disabled"'>
				<xsl:attribute name='disabled' />
			</xsl:if>
			<div class='formButtonCaption'>
				<u>S</u>ave
			</div>
		</button>
	</xsl:if>


	<xsl:if test='not(@close="no")'>
		<button 
			type='button' 
			class='button' 
			id='cmdClose{$formId}'
			onclick='Form.fnClose()'
			accesskey='C'>
			<xsl:if test='@close="disabled"'>
				<xsl:attribute name='disabled' />
			</xsl:if>
			<div class='formButtonCaption'>
				<u>C</u>lose
			</div>
		</button>
	</xsl:if>

	<xsl:if test='@ok="enabled" or @ok="disabled"'>
		<input 
			type='submit'
			class='button'
			id='cmdOK{$formId}'
			value='      OK      '
			accesskey='O'>
			<xsl:if test='@ok="disabled"'>
				<xsl:attribute name='disabled' />
			</xsl:if>
		</input>
	</xsl:if>

	<xsl:if test='@cancel="enabled" or @cancel="disabled"'>
		<button 
      type='button' 
			class='button' 
			id='cmdCancel{$formId}'
			onclick='Form.fnClose()'
			accesskey='C'>
			<xsl:if test='@cancel="disabled"'>
				<xsl:attribute name='disabled' />
			</xsl:if>
			<div class='formButtonCaption'>
				<u>C</u>ancel
			</div>
		</button>
	</xsl:if>

	<xsl:if test='not(@delete="no")'>
		<button 
      type='button' 
			class='button' 
			id='cmdDelete{$formId}'
			onclick='Form.fnDelete()'
			accesskey='D'>
			<xsl:if test='@delete="disabled"'>
				<xsl:attribute name='disabled' />
			</xsl:if>
			<div class='formButtonCaption'>
				<u>D</u>elete
			</div>
		</button>
	</xsl:if>

	<xsl:if test='not(@print="no")'>
		<button 
      type='button' 
			class='button' 
			id='cmdPrint{$formId}'
			onclick='window.print()'
			accesskey='P'>
			<xsl:if test='@print="disabled"'>
				<xsl:attribute name='disabled' />
			</xsl:if>
			<div class='formButtonCaption'>
				<u>P</u>rint
			</div>
		</button>
	</xsl:if>

	<xsl:if test='not(@help="no")'>
		<button 
      type='button' 
			class='button' 
			id='cmdHelp{$formId}'
			onclick='Form.fnHelp(event)'
			accesskey='H'>
			<xsl:if test='@help="disabled"'>
				<xsl:attribute name='disabled' />
			</xsl:if>
			<div class='formButtonCaption'>
				<u>H</u>elp
			</div>
		</button>
	</xsl:if>
</xsl:template>


<!-- Custom buttons to right of main button bar -->
<!-- Override to provide content -->
<xsl:template name='gjs:buttonBarAfter'>
</xsl:template>


<xsl:template match="gjs:frame" mode="form">

  <table
   class='formLayoutTable'
   style='margin: 20px; overflow: visible;'
   border='0'>

    <!-- Apply column widths and form fields -->
    <xsl:apply-templates mode='form' />

  </table>

  <xsl:apply-templates select='gjs:inputHidden' />
</xsl:template>


<xsl:template match="gjs:information" mode='form'>
  <!-- Table containing Info caption -->
  <table
   class='formLayoutTable'
   style='margin: 20px; margin-bottom: 0px'
   border='0'>
    <tr>
      <td class='caption'>
        <img src='vcm_info.gif' align='left' alt='Information' ondragstart='return false;'/>
	      <xsl:value-of select="."/>
      </td>
    </tr>
  </table>
</xsl:template>


<!-- First-generation tabstrip, which can only contain first-gen tabs. These do not support hint, nor i18n stuff.

Example:
  <gjs:tabStrip>
    <gjs:tabStripTab
     id='tabChangeRequest'
     accesskey='N'
     divId='divChangeRequest'>
      Mai<u>n</u> Details
    </gjs:tabStripTab>
    <gjs:tabStripTab
     id='tabComments'
     accesskey='M'
     divId='divComments'>
      Co<u>m</u>ments
    </gjs:tabStripTab>
  </gjs:tabStrip>
 -->
<xsl:template match="gjs:tabStrip" mode='form'>
  <!-- Table containing tab-strip -->
  <table
   class='tabStripLayoutTable'
   border='0'>
    <tr>
      <xsl:apply-templates select='gjs:tabStripTab' mode='form'/>
      <td 
       width='*'
       class='tabStripTabNone'>
      </td>
    </tr>
  </table>
</xsl:template>


<!-- First-generation tabstrip tab. Does not support hint, nor i18n stuff. -->
<xsl:template match="gjs:tabStripTab" mode='form'>


  <!-- Cell containing the tabstrip tab -->
  <td
   id='{@id}{$formId}'
   tabindex='-1'
   onfocus='TabStrip.fnSelect(this);'
   onkeydown='TabStrip.fnKeyDown(this,event);'
   gjsDivId='{@divId}'
   accesskey='{@accesskey}'
   class='tabStripTab'>
   <xsl:attribute name='width'>
     <xsl:value-of select="@width"/>
     <xsl:if test="not(@width)">100</xsl:if>
   </xsl:attribute>
    <span
     class='caption'>
      <xsl:copy-of select='.'/>   <!-- This isn't quite right -->
    </span>
  </td>
</xsl:template>


<!-- Second-generation tabstrip, supporting second-gen tabs.
For back-compatibility of VC/m custom change request tab feature, a second-gen tabstrip
may contain first-gen tabs. Regardless of sequence in the XML source, all second-gen ones precede all first-gen ones.

Example:
  <gjs:tabStrip2>
    <gjs:tabStripTab2
     id='tabChangeRequest'
     divId='divChangeRequest'>
       <label>Mai<u>n</u> Details</label>
       <hint>Main properties of the change request.</hint>
    </gjs:tabStripTab2>
    <gjs:tabStripTab2
     id='tabComments'
     divId='divComments'>
       <label>Co<u>m</u>ments</label>
       <hint>Comments and creation/amendment audit fields. Bold caption indicates existence of comment text.</hint>
    </gjs:tabStripTab2>
  </gjs:tabStrip2>
-->
<xsl:template match="gjs:tabStrip2" mode='form'>
  <!-- Table containing tab-strip -->
  <table
   class='tabStripLayoutTable'
   border='0'>
    <tr>
      <!-- Add the second-gen tabs first, then any first-gen ones -->
      <xsl:apply-templates select='gjs:tabStripTab2' mode='form'/>
      <xsl:apply-templates select='gjs:tabStripTab' mode='form'/>
      <td 
       width='*'
       class='tabStripTabNone'>
      </td>
    </tr>
  </table>
</xsl:template>


<!-- Second-generation tabstrip tab -->
<xsl:template match="gjs:tabStripTab2" mode='form'>
  <xsl:param name='tabId' select='@id'/>

  <xsl:variable name='label'>
    <xsl:call-template name='gjs:translateString'>
      <xsl:with-param name='string' select='@id'/>
      <xsl:with-param name='default' select='label'/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name='accesskey'>
    <xsl:call-template name='gjs:translateAccessKey'>
      <xsl:with-param name='string' select='@id'/>
      <xsl:with-param name='default' select='label'/>
      <xsl:with-param name='accessKey' select='@accesskey'/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name='hint'>
    <xsl:call-template name='gjs:translateString'>
      <xsl:with-param name='string' select='concat(@id,"_hint")'/>
      <xsl:with-param name='default' select='hint'/>
    </xsl:call-template>
  </xsl:variable>

  <!-- Cell containing the tabstrip tab -->
  <td
   id='{$tabId}'
   tabindex='-1'
   onfocus='TabStrip.fnSelect(this)'
   onkeydown='TabStrip.fnKeyDown(this,event)'
   gjsDivId='{@divId}'
   accesskey='{$accesskey}'
   title='{$hint}'
   class='tabStripTab'>
   <xsl:attribute name='width'>
     <xsl:value-of select="@width"/>
     <xsl:if test="not(@width)">100</xsl:if>
   </xsl:attribute>
    <span
     class='caption'>
      <xsl:copy-of select='$label'/>
    </span>
  </td>
</xsl:template>


<!-- Hidden Input fields -->
<xsl:template match="gjs:inputHidden">
      <input 
        type='hidden' 
        id='{@id}'
        name='{@id}'
        value='{@value}'/>
</xsl:template>


<xsl:template match="gjs:fields" mode='form'>
  <xsl:apply-templates/>
</xsl:template>


<xsl:template match="gjs:columnWidths" mode='form'>
  <tr>
    <xsl:apply-templates mode='form'/>
  </tr>
</xsl:template>


<xsl:template match="gjs:column" mode='form'>

	<xsl:variable name='width'>
		<xsl:choose>
			<xsl:when test='translate(@width,"px","")=@width'>
				<xsl:value-of select='concat(@width,"px")'/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select='@width'/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>

      <td class='caption' style='min-width: {$width}; height: 0px;'> </td>
</xsl:template>


</xsl:stylesheet>
