// ==UserScript==
// @name        Deltanji :: Production plugin
// @description Deltanji source control for Interoperability Productions
// @author      George James Software
// @namespace   https://georgejames.com
// @match       *://*/csp/*/EnsPortal.ProductionConfig.zen*
// @version     3
// ==/UserScript==


// Deltanji Connection
const deltanjiLocations = {}

// Deltanji Transfer Window
let deltanjiTransferWindow = null
let deltanjiTransferWindowOpen = false

// Name of Deltanji Client
const deltanjiClient = 'Client.VCm.cls'

// Highlight colours
const deltanjiRed='#f6ada7'
const deltanjiAmber='' // '#ffe098'
const deltanjiGreen='#ace4cc'

// When document has finished loading run the waitForZenPage() function
ready(waitForZenPage)


function ready(readyFunction) {
  if (document.readyState !== 'loading') readyFunction()
  else document.addEventListener('DOMContentLoaded', readyFunction)
}


// Now wait for the zenPage object to be ready then initialize by running init()
function waitForZenPage() {
  if (eval('zenPage') === null) setTimeout(waitForZenPage, 10)
  else init()
}


function init() {
  // hideSourceControlButtons()

  addDeltanjiButton()

  watchSvgDiagram()

  decorateItems()

  attachToDeleteButton()

  // Attach an event handler to the Expand/Contract all button so that we can restore the read-only'ness
  // after the user has toggled it
  document.querySelector('#btnExpandAll img').addEventListener('mouseup', updateSettingsTab)

  // Finally, make the source control button visible to indicate that everything loaded ok and we are ready to roll
  document.getElementById('deltanjiButton').setAttribute('src', productionIcon())
}


function attachToDeleteButton() {
  document.querySelector('#btnDelete input').addEventListener('click', function(element) {

	// Don't try to unregister if Deltanji has not been enabled
    if (!isDeltanjiEnabled()) return

    // If the item was successfully deleted then there should now be nothing selected
    const items = getSelectedItems()
    if (items.length !== 0) return

    const itemCret = document.getElementById('deltanjiButton').getAttribute('data-itemCret')
    if (itemCret === '') return
    
    const unregisterXhr = new XMLHttpRequest()
    unregisterXhr.onreadystatechange = function () {
        if (this.readyState == 4 && this.status == 200) {
            if (!handleErrorResponse(unregisterXhr.response)) return

            const status = unregisterXhr.responseText.split('\u0002')[0]
            const error = unregisterXhr.responseText.split('\u0002')[1]
            if (status === 'noop') return
            if (status === '1') {alert(itemCret + ' deleted from Deltanji source control'); return}
            alert(error)
        }
    }
    // Try to unregister the component
    unregisterXhr.open("GET", storageGet('deltanjiPath') + deltanjiClient + '?page=Production.fnUnregister&namespace=' + getNamespace() + '&cret=' + itemCret, true)
    unregisterXhr.send()

  })
}


// We don't want the traditional source control buttons to be displayed even if the Ensemble Source Control class is enabled
// Not currently used
function hideSourceControlButtons() {
  document.getElementById('sourceControlIcon').parentNode.setAttribute('style', 'display: none')
  document.getElementById('sourceControlNewOutputIcon').parentNode.setAttribute('style', 'display: none')

  // Adjust sizes twice otherwise it doesn't do it properly
  window.eval('zenPage.adjustSizes(true,true); zenPage.adjustSizes(true,true)')

}


// Add the Deltanji source control button to the top left corner of the Settings panel
function addDeltanjiButton() {

  // Create our source control button and add it to the page
  const deltanjiButtonHtml = '<div id="deltanjiButtonWrapper" style="float: left;">'
                           + '  <img tabindex="0" '
                           + '       id="deltanjiButton" '
                           + '       src="" '
                           + '       style="cursor: pointer; height: 31px; margin-left: 8px; margin-right: 18px; margin-top: 8px; margin-bottom: 5px;"> '
                           + '  </img> '
                           + '</div> '
  const propertyTitle = document.getElementById('propertyTitle')
  const propertyTitleHtml = propertyTitle.innerHTML
  propertyTitle.innerHTML = `${deltanjiButtonHtml} ${propertyTitleHtml}`
  
  // When the Production page is first loaded the Production Settings are shown by default
  // so add the component reference info and tooltip to our source control button
  const itemCret = getProductionCret()
  const deltanjiButton = document.getElementById('deltanjiButton')
  deltanjiButton.setAttribute('data-itemCret', itemCret)
  deltanjiButton.setAttribute('data-itemStatus', 2)
  deltanjiButton.setAttribute('title', 'Invoke Deltanji source control for the Production Class\n' + itemCret)

  // Attach event handlers to make the source control button highlight on mouseover
  document.getElementById('deltanjiButtonWrapper').addEventListener('mouseover', () => {
	  document.getElementById('deltanjiButton').setAttribute('src', productionIconHover())
  })
  document.getElementById('deltanjiButtonWrapper').addEventListener('mouseout', () => {
	  document.getElementById('deltanjiButton').setAttribute('src', productionIcon())
  })

  // Create onclick event handler for source control button to launch the Deltanji Transfer dialogue
  document.getElementById('deltanjiButtonWrapper').addEventListener('click', deltanjiButtonClick )

}


// Onclick event handler for source control button (to launch the Deltanji Transfer dialogue)
function deltanjiButtonClick(event) {

  // Ctrl+Click accesses the setup
  if (event.ctrlKey) { if (!configure()) return }

  // Do nothing unless Deltanji is enabled
  if (!isDeltanjiEnabled()) { if (!configure()) return }

  let functionCode = ''
  let fromLocation = ''
  let toLocation = ''

  // Derive default values for transfer dialogue
  const itemStatus = document.getElementById('deltanjiButton').getAttribute('data-itemStatus')
  if (itemStatus === '0') {
    functionCode = deltanjiLocations.outFunction
    fromLocation = deltanjiLocations.outFromLocation
    toLocation = deltanjiLocations.outToLocation
  }
  if (itemStatus === '1') {
    functionCode = deltanjiLocations.inFunction
    fromLocation = deltanjiLocations.inFromLocation
    toLocation = deltanjiLocations.inToLocation
  }
  if (itemStatus === '2') {
    functionCode = deltanjiLocations.registerFunction
    fromLocation = deltanjiLocations.registerFromLocation
    toLocation = deltanjiLocations.registerToLocation
  }
  if (itemStatus === 'various') {
    functionCode = "XFER"
    fromLocation = deltanjiLocations.inFromLocation
    toLocation = ""
  }

  const items = getSelectedItems()

  // Create $c(2) delimited list of selected items
  let cret = ''
  for (let i=0 ; i < items.length ; i++) {
    const item = items[i]

    const itemCret = getCret(item)
    if (cret !== '') cret = cret + '\u0002'
    cret = cret + itemCret
  }

  // Nothing selected so must be the Production Settings so cret is the Production class itself
  if (items.length === 0) {
    cret = getProductionCret()
  }

  // Single component to transfer
  if (items.length === 1) {

    // Single item
    // Open transfer dialogue window
    const url = storageGet('deltanjiPath') + deltanjiClient
              + '?page=Component.fnTransfer'
              + '&function=' + functionCode
              + '&lcode1=' + fromLocation
              + '&lcode2=' + toLocation
              + '&cret=' + cret //.split('\u0002')[0]
    deltanjiTransferWindow = window.open(url, 'DeltanjiTransfer', 'left=100,top=100,width=636,height=317')
    deltanjiTransferWindowOpen = true

    // Watch for when the Transfer Dialog gets closed
    windowWatcher()

  }
  else {
    // Multiple items
    const cretCount = cret.split('\u0002').length
    const crets = cretCount.toString() + '\u0002' + cret

    const selectUrl = storageGet('deltanjiPath') + deltanjiClient
    const selectRequest = 'page=Selection.fnSave'
                        + '&arguments='
                        + 'COMPONENT' + '\u0001'
                        + crets + '\u0001'
                        + '\u0001'

    const selectXhr = new XMLHttpRequest()
    selectXhr.onreadystatechange = function () {
      if (this.readyState === 4 && this.status !== 200) {
        alert('Cannot access Deltanji: Error ' + this.status)
        return
      }
      if (this.readyState == 4 && this.status == 200) {

        // Note handlerArgument is escaped so \002 needs to be referenced as \\002
        const handlerArgument = selectXhr.responseXML.documentElement.getElementsByTagName('HandlerArgument')[0].childNodes[0].nodeValue
        const status = handlerArgument.split('\\002')[0]
        if (status === 'ok') {

          // Open transfer dialogue window
          const selectionId = handlerArgument.split('\\002')[2]
          const url = storageGet('deltanjiPath') + deltanjiClient
                    + '?page=Component.fnTransfer'
                    + '&function=' + functionCode
                    + '&lcode1=' + fromLocation
                    + '&lcode2=' + toLocation
                    + '&obvType=SEL'
                    + '&obv=' + selectionId
          deltanjiTransferWindow = window.open(url, 'DeltanjiTransfer', 'left=100,top=100,width=636,height=317')
          deltanjiTransferWindowOpen = true

          // Watch for when the Transfer Dialog gets closed
          windowWatcher()
        }
      }
    }

    selectXhr.open('POST', selectUrl, true)
    selectXhr.setRequestHeader('Content-type', 'application/x-www-form-urlencoded')
    selectXhr.send(selectRequest)

  }
}


// Storage
function storageGet(key) {
  // Settings are specific to each namespace on each server instance
  // NB http://127.0.0.1 is different to http://localhost for configuration purposes
  const fullKey = document.location.origin + '|' + getNamespace() + '|' + key
  return localStorage.getItem(fullKey)
}


// Storage
function storageSet(key, value) {
  const fullKey = document.location.origin + '|' + getNamespace() + '|' + key
  localStorage.setItem(fullKey, value)
}


// Is Deltanji Enabled?
function isDeltanjiEnabled() {
  if (storageGet('deltanjiEnabled') === 'yes') return true
  return false
}


// Configure Deltanji for this namespace
function configure() {

  const deltanjiEnabled = storageGet('deltanjiEnabled')

  // Not enabled
  if (deltanjiEnabled !== 'yes') {
    const enable = confirm('The Deltanji userscript extension is not enabled for this namespace.  Click OK to enable.')
	if (!enable) {
      storageSet('deltanjiEnabled', 'no')
	  return false
    }
  }

  // Get Path
  let currentPath = storageGet('deltanjiPath')
  if (currentPath === null) {
    currentPath = document.location.origin + '/deltanji/'
  }

  // Allow the user to change it to something else
  let newPath = prompt("Enter the URL for the Deltanji server",currentPath)
  if (newPath === null) return false

  // Remove Client.VCm.cls if the user has entered it
  newPath = newPath.split('Client.VCm.cls')[0]

  // Validate the url by making a request to Production.fnStatus
  const productionCret = getProductionCret()

  const statusXhr = new XMLHttpRequest()
  statusXhr.onreadystatechange = function () {
    if (this.readyState === 4 && this.status !== 200) {
	  alert('Cannot access Deltanji: Error ' + this.status)
      return
    }
    if (this.readyState == 4 && this.status == 200) {
      if (!handleErrorResponse(statusXhr.response)) return

      // If we get back html instead of xml then Deltanji failed to authenticate but we don't
      // care about that at this point

      // Set the path and enable userscript
      storageSet('deltanjiPath', newPath)
      storageSet('deltanjiEnabled', 'yes')

      // Finally update everything
      decorateItems()
    }
  }

  // Get status of production
  statusXhr.open('GET', newPath + deltanjiClient + '?page=Production.fnStatus&namespace=' + getNamespace() + '&cret=' + productionCret, true)
  statusXhr.send()

  return false
}


// Watch the svg diagram for any changes
function watchSvgDiagram() {

  // Watch the svg element for any changes.  If there's a change then re-decorate the
  // items with our colour coding
  let svgWatcher = new MutationObserver(decorateItems)
  svgWatcher.observe(svgElement(), {childList: true, attributes: true} )

}


function updateSettingsTab() {

  setTimeout(updateSelectedItems, 100)

  // Do it a second time in case the first time was too soon
  setTimeout(updateSelectedItems, 1000)
}


// Update the Settings tab to read-only if the status is not-checked-out
function updateSelectedItems() {

  setDeltanjiButtonStatus()

  // How many items are selected?
  const items = getSelectedItems()
  const selectedItemCount = items.length

  let itemNo = '' 
  let itemStatus = ''

  // No items selected so assume Production Settings are selected
  if (selectedItemCount === 0) {
    itemNo = -1
    itemStatus = document.querySelector('a.dgmAction[title="Display general settings for the Production"]').getAttribute('data-itemStatus')
    itemStatus = document.querySelector('a.dgmAction[title="Display general settings for the Production"]').getAttribute('data-itemStatus')
  }

  // Just one item selected
  if (selectedItemCount === 1) {
    const item = items[0]
    itemNo = item.id.split('_')[1]
    itemStatus = item.getAttribute('data-itemStatus')
  }

  // Multiple items selected (do nothing)
  if (selectedItemCount > 1) { 
    itemNo = ''
    itemStatus = ''
  }

  if (itemStatus === '0') setItemReadonly(itemNo, true)  // Not checked-out, set to read-only
  if (itemStatus === '1') setItemReadonly(itemNo, false) // Checked-out, set to writeable
  if (itemStatus === '2') setItemReadonly(itemNo, false) // Unregistered, set to writeable
}


// Set the input fields in the Settings tab to readonly or read/write
function setItemReadonly(itemNo,readonly) {

  // Call the host page's updateSettings method, but first set the canEdit property to control its behaviour
  if (readonly) {
    window.eval("zenPage.canEdit=false; zenPage.updateSettings(zenPage.getHost('"+ itemNo +"')); zenPage.canEdit=true")
    document.querySelector('#btnDelete input').setAttribute('disabled', true)
    document.querySelector('#btnDelete input').setAttribute('class', 'commandButtonDisabled')
    document.querySelector('#btnChangeClass input').setAttribute('disabled', true)
    document.querySelector('#btnChangeClass input').setAttribute('class', 'commandButtonDisabled')
  }
  else {
    window.eval("zenPage.canEdit=true; zenPage.updateSettings(zenPage.getHost('"+ itemNo +"')); zenPage.canEdit=true")
    document.querySelector('#btnDelete input').setAttribute('disabled', false)
    document.querySelector('#btnDelete input').setAttribute('class', 'commandButton')
    document.querySelector('#btnChangeClass input').setAttribute('disabled', false)
    document.querySelector('#btnChangeClass input').setAttribute('class', 'commandButton')
  }

}


// Watch the Deltanji Transfer window, when it is closed then it will go Dead.  We then re-decorate the
// items to reflect any changes caused by the transfer
function windowWatcher() {
  if (deltanjiTransferWindowOpen && (isWindowDead())) {
    deltanjiTransferWindowOpen = false

    // If the underlying class has been modified (eg checked-out) then the IDs of Configuration Items will have changed
    // Performing a reload of the diagram will refetch these from the server, otherwise the page will get errors.
    window.eval('zenPage.reloadDiagram()')

    decorateItems()
    updateSettingsTab()
  }
  else setTimeout(windowWatcher, 100)
}


// Is the Deltanji Transfer window dead?
function isWindowDead() {
  // This will throw an error when the window has gone away
  try {
    return deltanjiTransferWindow.closed
  }
  catch { return true }
}


function getNamespace() {
  const folders = document.location.pathname.split('/')
  return folders[folders.length - 2].toUpperCase()
}


// Iterate each item in the production, get its status and decorate accordingly
function decorateItems() {

  // If Deltanji not enabled or configured then do nothing
  if (!isDeltanjiEnabled()) { return }

  const productionCret = getProductionCret()

  const statusXhr = new XMLHttpRequest()
  statusXhr.onreadystatechange = function () {
    if (this.readyState === 4 && this.status !== 200) {
      alert('Cannot access Deltanji: Error ' + this.status)
      return
    }
    if (this.readyState == 4 && this.status == 200) {

      // If we get back html instead of xml then Deltanji failed to authenticate
      if ((statusXhr.responseXML === null) && (statusXhr.response.includes('<html>'))) {
        alert('Deltanji authentication failed.  The Session Cookie Path for Web Application /csp/' + getNamespace() + ' and the Deltanji Web Application must both be set to /')
        return
      }

      if (!handleErrorResponse(statusXhr.response)) return

      const status = statusXhr.responseXML.getElementById('status')
      const excludeFromSourceControl = status.getAttribute('excludeFromSourceControl')
      if (excludeFromSourceControl === '0') {
        alert('To use the Deltanji Production plugin for this namespace you should disable normal class based source control. \n\nNavigate to Interoperability > Manage > Configuration > Interoperability Settings. Check the box labelled Exclude Production Configuration Page from Source Control and then click Apply.')
      }

      const locations = statusXhr.responseXML.getElementById('locations')
      deltanjiLocations.getFunction = locations.getAttribute('getFunction')
      deltanjiLocations.getFromLocation = locations.getAttribute('getFromLocation')
      deltanjiLocations.getToLocation = locations.getAttribute('getToLocation')
      deltanjiLocations.inFunction = locations.getAttribute('inFunction')
      deltanjiLocations.inFromLocation = locations.getAttribute('inFromLocation')
      deltanjiLocations.inToLocation = locations.getAttribute('inToLocation')
      deltanjiLocations.outFunction = locations.getAttribute('outFunction')
      deltanjiLocations.outFromLocation = locations.getAttribute('outFromLocation')
      deltanjiLocations.outToLocation = locations.getAttribute('outToLocation')
      deltanjiLocations.registerFunction = locations.getAttribute('registerFunction')
      deltanjiLocations.registerFromLocation = locations.getAttribute('registerFromLocation')
      deltanjiLocations.registerToLocation = locations.getAttribute('registerToLocation')

      // Get status of Production Class
      const productionClassStatus = statusXhr.responseXML.getElementById(productionCret).getAttribute('status')

      // Set background colour
      var colour = deltanjiRed
      if (productionClassStatus === '0') { colour = deltanjiAmber }
      if (productionClassStatus === '1') { colour = deltanjiGreen }
      if (productionClassStatus === '2') { colour = deltanjiRed }

      // Decorate the Production class name and make it clickable
      const classNameCaption = document.getElementById('pageTitleText')
      const productionId = getProductionId()
      classNameCaption.innerHTML = `<span id="deltanjiProduction" data-productionClassStatus="${productionClassStatus}" style="cursor: pointer; background-color: ${colour}; border-radius: 3px;"> ${productionId} </span>`
      classNameCaption.addEventListener('mouseup', transferProduction)

      // Decorate the Production Settings caption
      const item = document.querySelector('a.dgmAction[title="Display general settings for the Production"]')
      decorateProductionSettings(item, productionClassStatus)

      // Iterate all items and decorate them according to their status
      for (let i=0 ; ; i++) {

        const item = svgElement().getSVGDocument().getElementById('item_'+i)
        if (item === null) break

        const itemCret = getCret(item)
        const itemElement = statusXhr.responseXML.getElementById(itemCret)
        if (itemElement !== null) {
          const itemStatus = itemElement.getAttribute('status')
          decorateItem(item, itemStatus)
        }
        else {
          decorateItem(item, 2) // Treat as unregistered
        }
      }

      updateSettingsTab()

    }
  }

  // Get status of production
  statusXhr.open('GET', storageGet('deltanjiPath') + deltanjiClient + '?page=Production.fnStatus&namespace=' + getNamespace() + '&cret=' + productionCret, true)
  statusXhr.send()

}


function transferProduction() {

  let functionCode = ''
  let fromLocation = ''
  let toLocation = ''
  const productionClassStatus = document.getElementById('deltanjiProduction').getAttribute('data-productionClassStatus')
  if (productionClassStatus === '0') {
    functionCode = deltanjiLocations.outFunction
    fromLocation = deltanjiLocations.outFromLocation
    toLocation = deltanjiLocations.outToLocation
  }
  if (productionClassStatus === '1') {
    functionCode = deltanjiLocations.inFunction
    fromLocation = deltanjiLocations.inFromLocation
    toLocation = deltanjiLocations.inToLocation
  }
  if (productionClassStatus === '2') {
    functionCode = deltanjiLocations.registerFunction
    fromLocation = deltanjiLocations.registerFromLocation
    toLocation = deltanjiLocations.registerToLocation
  }

  const cret = getProductionCret()
  const url = storageGet('deltanjiPath') + deltanjiClient
            + '?page=Component.fnTransfer'
            + '&function=' + functionCode
            + '&lcode1=' + fromLocation
            + '&lcode2=' + toLocation
            + '&cret=' + cret
  deltanjiTransferWindow = window.open(url, 'DeltanjiTransfer', 'left=100,top=100,width=636,height=317')
  deltanjiTransferWindowOpen = true

  // Watch for when the Transfer Dialog gets closed
  windowWatcher()
}


// If response contains a script element then evaluate it and return false
function handleErrorResponse(response) {
  if (response.substring(0,8) === '<script>') {
    const javascript = response.substring(8, response.length-11)
    eval(javascript)
    return false
  }
  return true
}


// Decorate an item based on its status and add a mouseup handler
function decorateItem(item, itemStatus) {

  var colour = deltanjiRed

  // Set background colour
  if (itemStatus === '0') { colour = deltanjiAmber }
  if (itemStatus === '1') { colour = deltanjiGreen }
  if (itemStatus === '2') { colour = deltanjiRed }
  item.setAttribute("style", 'fill:' + colour)
  item.setAttribute("data-itemStatus", itemStatus)

  // Add mouseup event listener
  item.addEventListener('mouseup', itemMouseUp)
}


// Decorate the Production Settings button based on its status and add a mouseup handler
function decorateProductionSettings(item, itemStatus) {

  var colour = deltanjiRed

  // Set background colour
  if (itemStatus === '0') { colour = deltanjiAmber }
  if (itemStatus === '1') { colour = deltanjiGreen }
  if (itemStatus === '2') { colour = deltanjiRed }
  item.setAttribute('style', 'padding-left: 5px; padding-right: 5px; background-color:' + colour + ';color: #041390; border-radius: 3px')
  item.setAttribute("data-itemStatus", itemStatus)

  // Add mouseup event listener
  item.addEventListener('mouseup', itemMouseUp)
}


// When an item is clicked then update the Deltanji source control button with details
// about the selected item, and set the contents of the settings tab to read or write
// depending on the status of the item
function itemMouseUp(event) {

  setDeltanjiButtonStatus()
  updateSettingsTab()

}


function setDeltanjiButtonStatus() {

  const deltanjiButton = document.getElementById('deltanjiButton')
  const items = getSelectedItems()
  const selectItemCount = items.length

  // No items selected so assume Production Settings are selected
  if (selectItemCount === 0) {
    const productionCret = getProductionCret()
    const productionClassStatus = document.querySelector('a.dgmAction[title="Display general settings for the Production"]').getAttribute('data-itemStatus')
    deltanjiButton.setAttribute('data-itemCret', productionCret)
    deltanjiButton.setAttribute('data-itemStatus', productionClassStatus)
    deltanjiButton.setAttribute('title', 'Invoke Deltanji source control for the Production Class\n' + productionCret )
    deltanjiButton.setAttribute('src', productionIcon())
  }

  // One item selected
  if (selectItemCount === 1) {
    const item = items[0]
    const itemCret = getCret(item)
    const itemStatus = item.getAttribute('data-itemStatus')
    deltanjiButton.setAttribute('data-itemCret', itemCret) // Used by deletion handler
    deltanjiButton.setAttribute('data-itemStatus', itemStatus)
    deltanjiButton.setAttribute('title', 'Invoke Deltanji source control for the selected Configuration Item\n' + itemCret)
    deltanjiButton.setAttribute('src', productionIcon())
  }

  // Multiple items selected
  if (selectItemCount > 1) {

    // Do all the selected items have the same status?
    let aggregateItemStatus = ''
    for (let i=0; i<items.length; i++) {
      const item = items[i]
      const itemStatus = item.getAttribute('data-itemStatus')
      if (aggregateItemStatus === '') aggregateItemStatus = itemStatus
      if (aggregateItemStatus !== itemStatus) aggregateItemStatus = 'various'
    }

    deltanjiButton.setAttribute('data-itemCret', '') // Used by deletion handler
    deltanjiButton.setAttribute('data-itemStatus', aggregateItemStatus)
    deltanjiButton.setAttribute('title', 'Invoke Deltanji source control for the selected Configuration Items\n' + items.length + ' items selected')
    deltanjiButton.setAttribute('src', productionIcon())
  }
}


// Get human readable status from status code
function getStatus(itemStatus) {
	if (itemStatus === '0') { return 'Checked-in' }
	if (itemStatus === '1') { return 'Checked-out' }
	if (itemStatus === '2') { return 'Unregistered' }
	return "Unknown"
}


// Derive the component reference of an item
function getCret(item) {

  const productionId = getProductionId()

  // Production Settings
  if (item.getAttribute('title') === "Display general settings for the Production") {
    return getProductionCret()
  }

  // Regular items
  const itemId = item.id
  const itemName = item.getAttribute('title')
  const itemClassName = window.eval('zenPage.getHost('+ itemId.split('_')[1] +').classname')

  let itemCret = ''
  if (itemName === itemClassName) { itemCret = 'PRI.' + productionId + '.' + itemName }
  else { itemCret = 'PRI.' + productionId + '.' + itemName + ' [' + itemClassName.replace(/\./g,"_") + ']' }

  return itemCret
}


// Get the cret for the Production class
function getProductionCret() {
  return "CLS." + getProductionId()
}


function getProductionId() {
  return window.eval('zenPage.productionId')
}


function productionIcon() {

  // If url not configured yet then return the built-in source control icon
  const deltanjiPath = storageGet('deltanjiPath')
  if (deltanjiPath === null) return 'deepsee/ds2_merge_44.png'

  const itemStatus = document.getElementById('deltanjiButton').getAttribute('data-itemStatus')
  if (itemStatus === '0') { return deltanjiPath + 'productionRegistered.png' }
  if (itemStatus === '1') { return deltanjiPath + 'productionCheckedOut.png' }
  if (itemStatus === '2') { return deltanjiPath + 'productionUnregistered.png' }
  if (itemStatus === 'various') { return deltanjiPath + 'productionMultiSelect.png' }
  return deltanjiPath + 'productionUnregistered.png'
}


function productionIconHover() {

  // If url not configured yet then return the built-in source control icon
  const deltanjiPath = storageGet('deltanjiPath')
  if (deltanjiPath === null) return 'deepsee/ds2_merge_44.png'

  const itemStatus = document.getElementById('deltanjiButton').getAttribute('data-itemStatus')
  if (itemStatus === '0') { return deltanjiPath + 'productionRegisteredHover.png' }
  if (itemStatus === '1') { return deltanjiPath + 'productionCheckedOutHover.png' }
  if (itemStatus === '2') { return deltanjiPath + 'productionUnregisteredHover.png' }
  if (itemStatus === 'various') { return deltanjiPath + 'productionMultiSelectHover.png' }
  return deltanjiPath + 'productionUnregisteredHover.png'
}


function svgElement() {
  return document.getElementById('svg').children[0].children[0]
}


// Get all selected items
function getSelectedItems() {

  // ChatGPT says: This code uses the Array.prototype.slice.call() method to 
  // convert the HTMLCollection returned by the getElementsByClassName() method into an array. The getElementsByClassName() 
  // method is used to get all elements with the class name 'itemRectSelect' and 'itemRectSelectHover'. The two arrays are 
  // then concatenated using the concat() method to create a single array of all selected items. The array is then returned 
  // by the function.
  const selectedItems = Array.prototype.slice.call( svgElement().getSVGDocument().getElementsByClassName('itemRectSelect'), 0)
  const selectedItemsHover = Array.prototype.slice.call( svgElement().getSVGDocument().getElementsByClassName('itemRectSelectHover'), 0)
  const allSelectedItems = selectedItems.concat(selectedItemsHover)
  return allSelectedItems
}
